@@ -1,17 +1,17 @@
 /*
     LinKNX KNX home automation platform
     Copyright (C) 2007 Jean-François Meessen <linknx@ouaye.net>
-
+ 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
-
+ 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
-
+ 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -56,7 +56,6 @@ public:
     virtual void setValue(ObjectValue* value) = 0;
 
     virtual void importXml(ticpp::Element* pConfig);
-    virtual void updateXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
 
     const char* getID() { return id_m.c_str(); };
@@ -100,9 +99,14 @@ public:
     virtual void setIntValue(int value) { setBoolValue(value != 0); };
     virtual void setFloatValue(float value) { setBoolValue(value != 0); };
     virtual void setBoolValue(bool value);
-    virtual bool getBoolValue() { if (!init_m) read(); return value_m; };
+    virtual bool getBoolValue()
+    {
+        if (!init_m)
+            read();
+        return value_m;
+    };
     virtual int getIntValue() { return getBoolValue() ? 1 : 0; };
-private:
+protected:
     bool value_m;
 };
 
@@ -119,10 +123,9 @@ public:
     virtual void exportXml(ticpp::Element* pConfig);
 
     virtual void onWrite(const uint8_t* buf, int len);
-    virtual void setIntValue(int value);
-    virtual int getIntValue() { if (!init_m) read(); return value_m; };
-private:
-    int value_m;
+protected:
+    int direction_m;
+    int stepcode_m;
 };
 
 class TimeObject : public Object
@@ -140,7 +143,7 @@ public:
     virtual void onWrite(const uint8_t* buf, int len);
     virtual void setTime(time_t time);
     virtual void setTime(int wday, int hour, int min, int sec);
-private:
+protected:
     int wday_m;
     int hour_m;
     int min_m;
@@ -162,7 +165,7 @@ public:
     virtual void onWrite(const uint8_t* buf, int len);
     virtual void setDate(time_t time);
     virtual void setDate(int day, int month, int year);
-private:
+protected:
     int day_m;
     int month_m;
     int year_m;
@@ -185,9 +188,14 @@ public:
     virtual void setFloatValue(float value);
     virtual void setBoolValue(bool value) { setFloatValue(value ? 1 : 0); };
     virtual int getIntValue() { return (int)getFloatValue(); };
-    virtual float getFloatValue() { if (!init_m) read(); return value_m; };
+    virtual float getFloatValue()
+    {
+        if (!init_m)
+            read();
+        return value_m;
+    };
     virtual bool getBoolValue() { return getFloatValue() != 0; };
-private:
+protected:
     float value_m;
 };
 
@@ -205,19 +213,31 @@ public:
 
     virtual void onWrite(const uint8_t* buf, int len);
     virtual void setIntValue(int value);
-    virtual int getIntValue() { if (!init_m) read(); return value_m; };
-private:
+    virtual int getIntValue()
+    {
+        if (!init_m)
+            read();
+        return value_m;
+    };
+protected:
     int value_m;
 };
 
+class HeatingModeObject : public ScalingObject
+{
+public:
+    virtual ObjectValue* createObjectValue(const std::string& value);
+    virtual void exportXml(ticpp::Element* pConfig);
+};
+
 class SwitchingObjectValue : public ObjectValue
 {
 public:
     SwitchingObjectValue(const std::string& value);
     virtual ~SwitchingObjectValue() {};
     virtual std::string toString();
-private:
-	friend class SwitchingObject;
+protected:
+    friend class SwitchingObject;
     bool value_m;
 };
 
@@ -227,9 +247,10 @@ public:
     DimmingObjectValue(const std::string& value);
     virtual ~DimmingObjectValue() {};
     virtual std::string toString();
-private:
-	friend class DimmingObject;
-    int value_m;
+protected:
+    friend class DimmingObject;
+    int direction_m;
+    int stepcode_m;
 };
 
 class TimeObjectValue : public ObjectValue
@@ -238,8 +259,8 @@ public:
     TimeObjectValue(const std::string& value);
     virtual ~TimeObjectValue() {};
     virtual std::string toString();
-private:
-	friend class TimeObject;
+protected:
+    friend class TimeObject;
     int wday_m;
     int hour_m;
     int min_m;
@@ -252,8 +273,8 @@ public:
     DateObjectValue(const std::string& value);
     virtual ~DateObjectValue() {};
     virtual std::string toString();
-private:
-	friend class DateObject;
+protected:
+    friend class DateObject;
     int day_m;
     int month_m;
     int year_m;
@@ -265,8 +286,8 @@ public:
     ValueObjectValue(const std::string& value);
     virtual ~ValueObjectValue() {};
     virtual std::string toString();
-private:
-	friend class ValueObject;
+protected:
+    friend class ValueObject;
     float value_m;
 };
 
@@ -276,16 +297,31 @@ public:
     ScalingObjectValue(const std::string& value);
     virtual ~ScalingObjectValue() {};
     virtual std::string toString();
-private:
-	friend class ScalingObject;
+protected:
+    ScalingObjectValue() {};
+    friend class ScalingObject;
     int value_m;
 };
 
+class HeatingModeObjectValue : public ScalingObjectValue
+{
+public:
+    HeatingModeObjectValue(const std::string& value);
+    virtual std::string toString();
+protected:
+    friend class HeatingModeObject;
+};
+
 class ObjectController : public TelegramListener
 {
 public:
     static ObjectController* instance();
-    static void reset() { if (instance_m) delete instance_m; instance_m = 0; };
+    static void reset()
+    {
+        if (instance_m)
+            delete instance_m;
+        instance_m = 0;
+    };
     void addObject(Object* object);
     void removeObject(Object* object);
 
