@@ -1,17 +1,17 @@
 /*
     LinKNX KNX home automation platform
     Copyright (C) 2007 Jean-François Meessen <linknx@ouaye.net>
-
+ 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
-
+ 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
-
+ 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -51,7 +51,7 @@ public:
     virtual bool evaluate();
     virtual void importXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
-    
+
 private:
     ConditionsList_t conditionsList_m;
     ChangeListener* cl_m;
@@ -66,7 +66,7 @@ public:
     virtual bool evaluate();
     virtual void importXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
-    
+
 private:
     ConditionsList_t conditionsList_m;
     ChangeListener* cl_m;
@@ -81,7 +81,7 @@ public:
     virtual bool evaluate();
     virtual void importXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
-    
+
 private:
     Condition* condition_m;
     ChangeListener* cl_m;
@@ -96,7 +96,7 @@ public:
     virtual bool evaluate();
     virtual void importXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
-    
+
 private:
     Object* object_m;
     ObjectValue* value_m;
@@ -200,6 +200,38 @@ private:
     std::string value_m;
 };
 
+class SendEmailAction : public Action
+{
+public:
+    SendEmailAction();
+    virtual ~SendEmailAction();
+
+    virtual void importXml(ticpp::Element* pConfig);
+    virtual void exportXml(ticpp::Element* pConfig);
+
+private:
+    virtual void Run (pth_sem_t * stop);
+
+    std::string to_m;
+    std::string subject_m;
+    std::string text_m;
+};
+
+class ShellCommandAction : public Action
+{
+public:
+    ShellCommandAction();
+    virtual ~ShellCommandAction();
+
+    virtual void importXml(ticpp::Element* pConfig);
+    virtual void exportXml(ticpp::Element* pConfig);
+
+private:
+    virtual void Run (pth_sem_t * stop);
+
+    std::string cmd_m;
+};
+
 class Rule : public ChangeListener
 {
 public:
@@ -214,7 +246,7 @@ public:
     virtual void onChange(Object* object);
 
     void evaluate();
-    
+
 private:
     std::string id_m;
     Condition* condition_m;
@@ -229,7 +261,12 @@ class RuleServer
 {
 public:
     static RuleServer* instance();
-    static void reset() { if (instance_m) delete instance_m; instance_m = 0; };
+    static void reset()
+    {
+        if (instance_m)
+            delete instance_m;
+        instance_m = 0;
+    };
 
     virtual void importXml(ticpp::Element* pConfig);
     virtual void exportXml(ticpp::Element* pConfig);
@@ -240,7 +277,6 @@ private:
     typedef std::pair<std::string ,Rule*> RuleIdPair_t;
     typedef std::map<std::string ,Rule*> RuleIdMap_t;
     RuleIdMap_t rulesMap_m;
-//    std::list<Rule*> rulesList_m;
     static RuleServer* instance_m;
 };
 
